@@ -46,6 +46,7 @@ SOFTWARE.
 #define LARGE_COORDINATE 1000000
 #define SMALL_COORDINATE -LARGE_COORDINATE
 
+#include <config.h>
 #include <gdkregion.h>
 #include "gdkregion-generic.h"
 #include "gdkpoly-generic.h"
@@ -538,6 +539,7 @@ gdk_region_polygon(GdkPoint *Pts, gint Count, GdkFillRule rule)
                  */
                 if (iPts == NUMPTSTOBUFFER) {
                     tmpPtBlock = (POINTBLOCK *)g_malloc(sizeof(POINTBLOCK));
+		    tmpPtBlock->next = NULL;
                     curPtBlock->next = tmpPtBlock;
                     curPtBlock = tmpPtBlock;
                     pts = curPtBlock->pts;
@@ -584,6 +586,7 @@ gdk_region_polygon(GdkPoint *Pts, gint Count, GdkFillRule rule)
                      */
                     if (iPts == NUMPTSTOBUFFER) {
                         tmpPtBlock = (POINTBLOCK *)g_malloc(sizeof(POINTBLOCK));
+			tmpPtBlock->next = NULL;
                         curPtBlock->next = tmpPtBlock;
                         curPtBlock = tmpPtBlock;
                         pts = curPtBlock->pts;
