@@ -29,7 +29,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include "config.h"
+#include <config.h>
 
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
@@ -1021,11 +1021,11 @@ gtk_calendar_main_button (GtkWidget	 *widget,
       if (!GTK_WIDGET_HAS_FOCUS (widget))
 	gtk_widget_grab_focus (widget);
 	  
-      gtk_calendar_select_and_focus_day (calendar, day);
-
       private_data->in_drag = 1;
       private_data->drag_start_x = x;
       private_data->drag_start_y = y;
+
+      gtk_calendar_select_and_focus_day (calendar, day);
     }
   else if (event->type == GDK_2BUTTON_PRESS)
     {
@@ -2747,10 +2747,10 @@ gtk_calendar_button_press (GtkWidget	  *widget,
 	  /* only call the action on single click, not double */
 	  if (event->type == GDK_BUTTON_PRESS)
 	    {
-	      arrow_action (calendar, arrow);
-	      
 	      if (event->button == 1)
 		start_spinning (widget, arrow);
+
+	      arrow_action (calendar, arrow);	      
 	    }
 
 	  return TRUE;
@@ -3066,7 +3066,10 @@ gtk_calendar_state_changed (GtkWidget	   *widget,
   private_data = GTK_CALENDAR_PRIVATE_DATA (widget);
   
   if (!GTK_WIDGET_IS_SENSITIVE (widget))
-    stop_spinning (widget);    
+    {
+      private_data->in_drag = 0;
+      stop_spinning (widget);    
+    }
 
   for (i = 0; i < 4; i++)
     if (GTK_WIDGET_IS_SENSITIVE (widget))
@@ -3368,7 +3371,7 @@ gtk_calendar_drag_data_get (GtkWidget        *widget,
   gchar str[128];
   gsize len;
 
-  date = g_date_new_dmy (calendar->selected_day, calendar->month, calendar->year);
+  date = g_date_new_dmy (calendar->selected_day, calendar->month + 1, calendar->year);
   len = g_date_strftime (str, 127, "%x", date);
   gtk_selection_data_set_text (selection_data, str, len);
   
@@ -3529,7 +3532,7 @@ gtk_calendar_drag_data_received (GtkWidget        *widget,
   g_object_freeze_notify (G_OBJECT (calendar));
   if (!(calendar->display_flags & GTK_CALENDAR_NO_MONTH_CHANGE)
       && (calendar->display_flags & GTK_CALENDAR_SHOW_HEADING))
-    gtk_calendar_select_month (calendar, month, year);
+    gtk_calendar_select_month (calendar, month - 1, year);
   gtk_calendar_select_day (calendar, day);
   g_object_thaw_notify (G_OBJECT (calendar));  
 }
